package collisionDetector.detector.motionCreator;

import collisionDetector.main.data.Constants;
import collisionDetector.main.dataStructures.Aircraft;
import collisionDetector.main.dataStructures.Vector3d;



/**
 * Objects of the <code>Motion</code> class provide all a-priori known information about what the Aircraft was doing
 * between two frames. Also provides methods to do useful things with this object (so it's not just dead data).
 * @author Filip Pizlo
 */
public class Motion {
    /** The aircraft that we are referring to. */
    private final Aircraft aircraft;
    /**
     * The first position (from the last frame). Will be equal to <code>pos_two</code> if we do not have a record for
     * this aircraft for any previous frames.
     */
    private final Vector3d pos_one;
    /** The second position (from the current frame). */
    private final Vector3d pos_two;

    /** Initialize with an aircraft and two positions. */
    public Motion(final Aircraft _aircraft, final Vector3d _pos_one, final Vector3d _pos_two) {
        aircraft = _aircraft;
        pos_one = _pos_one;
        pos_two = _pos_two;
    }

    /** Initialize with an aircraft and one position. */
    public Motion(final Aircraft _aircraft, final Vector3d _pos) {
        this(_aircraft, _pos, _pos);
    }

    /** Retrieve the aircraft. */
    public Aircraft getAircraft() {
        return aircraft;
    }

    /** Retrieve position #1. */
    public Vector3d getFirstPosition() {
        return pos_one;
    }

    /** Retrieve position #2. */
    public Vector3d getSecondPosition() {
        return pos_two;
    }

    /**
     * Find an intersection between this Motion and another.
     * @return a Vector3d object with the intersection point if an intersection was found, null otherwise.
     * @author Jeff Hagelberg, Filip Pizlo
     */
    public Vector3d findIntersection(final Motion other) {
        final Vector3d i1 = new Vector3d(), f1 = new Vector3d(), i2 = new Vector3d(), f2 = new Vector3d();
        i1.set(getFirstPosition());
        f1.set(getSecondPosition());
        i2.set(other.getFirstPosition());
        f2.set(other.getSecondPosition());
        float r = Constants.PROXIMITY_RADIUS;
        final float vx1 = f1.x - i1.x;
        final float vx2 = f2.x - i2.x;
        final float vy1 = f1.y - i1.y;
        final float vy2 = f2.y - i2.y;
        final float vz1 = f1.z - i1.z;
        final float vz2 = f2.z - i2.z;

        // if a = 0, the planes are moving in parallel (or not moving at all)
        final float a = (vx2 - vx1) * (vx2 - vx1) + (vy2 - vy1) * (vy2 - vy1) + (vz2 - vz1) * (vz2 - vz1);
        float b = 2.0f * (i2.x * vx2 - i2.x * vx1 - i1.x * vx2 + i1.x * vx1 + i2.y * vy2 - i2.y * vy1 - i1.y * vy2
                + i1.y * vy1 + i2.z * vz2 - i2.z * vz1 - i1.z * vz2 + i1.z * vz1);
        final float c = -r * r + (i2.x - i1.x) * (i2.x - i1.x) + (i2.y - i1.y) * (i2.y - i1.y) + (i2.z - i1.z)
                * (i2.z - i1.z);

        if (a != 0.0f) {
            final float descr = b * b - 4.0f * a * c;
            if (descr < 0.0f) return null;

            // the left side
            final float v1 = (-b - (float) Math.sqrt(descr)) / (2.0f * a);
            // the right side
            final float v2 = (-b + (float) Math.sqrt(descr)) / (2.0f * a);

            if (v1 <= v2 && (v1 <= 1.0f && 1.0f <= v2 || v1 <= 0.0f && 0.0f <= v2 || 0.0f <= v1 && v2 <= 1.0f)) {
                // new: calculate the location of the collision; if it is
                // outside of the bounds of the Simulation, don't do anything!
                final float x1col = i1.x + vx1 * (v1 + v2) / 2.0f;
                final float y1col = i1.y + vy1 * (v1 + v2) / 2.0f;
                final float z1col = i1.z + vz1 * (v1 + v2) / 2.0f;
                if (z1col > Constants.MIN_Z && z1col <= Constants.MAX_Z && x1col >= Constants.MIN_X
                        && x1col <= Constants.MAX_X && y1col >= Constants.MIN_Y && y1col <= Constants.MAX_Y) return new Vector3d(
                        x1col, y1col, z1col);
            }
        } else {
            float dist = (i2.x - i1.x) * (i2.x - i1.x) + (i2.y - i1.y) * (i2.y - i1.y) + (i2.z - i1.z) * (i2.z - i1.z);
            dist = (float) Math.sqrt(dist);
            // System.out.println("i1 = "+i1+", i2 = "+i2+", dist = "+dist);
            if (dist <= r)
            // System.out.println("Planes were travelling in parallel. Collision.");
            return getFirstPosition();
        }
        return null;
    }
}
