package collisionDetector.simulator;

import collisionDetector.detector.reciever.IReceiver;
import collisionDetector.main.Start;
import collisionDetector.main.data.Constants;
import collisionDetector.main.dataStructures.Vector3d;
import collisionDetector.simulator.data.SimulatorConstants;







/**
 * The <code>BaseSimulator</code> class implements the functionality of driving the simulation. It manages the
 * locations of aircrafts in the simulation and updates their positions as the simulation executes. All units are
 * kilometers.
 * @author Ben L. Titzer
 */
public class MySimulator {

    public static int position;
    public static int counter;

    private IReceiver iReceiver;
    
    
    
    public static class Parameters {
        public int fps;
        public int max_craft;
        public int max_frames;
        public SimulationRegion region;
        public float timescale;
        public SimAircraftProducer director;
        public SimAircraftProducer producer;
    }

    private final int fps;
    private final int max_craft;
    private final SimulationRegion region;
    private final float timescale;
    private final Vector3d vec0 = new Vector3d(); // temporary
    private final SimAircraftProducer director;
    private final SimAircraftProducer producer;
    private double current_time;

    
    private static SerializedFrame[] frames;
    
    /**
     * The <code>BaseSimulator</code> class has a constructor that takes an instance of the
     * <code>BaseSimulator.Parameters</code> class that encapsulate the settings for the new simulator being created.
     * @param p the settings for the simulator instance to be created.
     */
    public MySimulator(final MySimulator.Parameters p) {
        fps = checkArgument(p.fps, SimulatorConstants.MIN_FPS, SimulatorConstants.MAX_FPS, "Invalid fps");
        max_craft = checkArgument(p.max_craft, SimulatorConstants.MIN_CRAFT, SimulatorConstants.MAX_CRAFT, "Invalid maximum craft count");
        region = p.region;
        timescale = checkArgument(p.timescale, 1, SimulatorConstants.MAX_TIMESCALE, "Invalid timescale");
        director = p.director;
        producer = p.producer;
        
    }

    private int checkArgument(final int val, final int min, final int max, final String error) {
        if (val < min || val > max) throw new IllegalArgumentException(error);
        return val;
    }

    private float checkArgument(final float val, final float min, final float max, final String error) {
        if (val < min || val > max) throw new IllegalArgumentException(error);
        return val;
    }

    /**
     * The <code>run</code> method of the simulator begins execution of the simulation with the parameters that were
     * supplied to the constructor. It executes the simulation until the maximum number of steps is reached or until the
     * <code>stop</code> method is called (by code reachable from the <code>run</code> method.
     */
    public void run() {
        run_();
        System.out.println("Simulation done in (" + (float) current_time + " secs).");
        for (int i = 0; i < Start.pos; i = i + 2) {
            System.out.println(i + ": " + (Start.times[i + 1] - Start.times[i]) / 1000000 + "  ms");
        }
    }

    /*
    class SerializedFrame {
      float[] positions;
      byte[] callsigns;
      int[] lengths;
      
      //@ales
      public void print() {
      	for(int i=0 ;i<positions.length;i++) {
      		System.out.print(positions[i]);
      	}
      	for(int i=0 ;i<callsigns.length;i++) {
      		System.out.print(callsigns[i]);
      	}
      	System.out.println();
      }
    }
    */
    

    private SerializedFrame translate(final SimFrame frame) {
        final SerializedFrame sf = new SerializedFrame();
        sf.positions = new float[3 * frame.used];
        int length = 0;
        for (int i = 0; i < frame.used; i++)
            length += frame.aircraft[i].callsign.length();
        sf.callsigns = new byte[length];
        sf.lengths = new int[frame.used];
        for (int i = 0, pos = 0; i < frame.used; i++) {
            final SimAircraft a = frame.aircraft[i];
            final byte[] b = a.callsign.getBytes();
            sf.lengths[i] = b.length;
            //for (final byte element : b)
            //    sf.callsigns[pos++] = element;
            
            for(int j=0; j < b.length ; j++) { 
            	sf.callsigns[pos++] = b[j];
            }
            
        }
        return sf;
    }

    private void run_() {
        // amount of time that passes between two frames.
        final double delta = timescale / fps;
        double next_time = 0;
        FrameFactory.Builder builder = SimMediator.frameFactory.getBuilder(max_craft, 0);
        producer.addInitialAircraft(region, builder);
        SimFrame prev = builder.makeFrame();
        final SerializedFrame sf = translate(prev);

       
        int loops = 1;
        int limit = Constants.MAX_FRAMES;
        //int limit = 4;
        //
       
        // Ales
        iReceiver.sendFromPJ(sf.positions, sf.lengths, sf.callsigns);
       
        
        frames[loops] = sf;
        sf.print();
        
        while (loops < limit) {
            next_time = current_time + delta;
            builder = SimMediator.frameFactory.getBuilder(max_craft, next_time);
            addRemainingAircraft(prev, builder);
            final FrameFactory.Mutator mutator = SimMediator.frameFactory.getMutator(builder);
            director.computeNewPositions(region, mutator, delta);
            mutator.commit(); // make sure the changes are finished
            producer.addAircraft(region, prev, builder, delta);
            final SimFrame next = builder.makeFrame();

            final SerializedFrame nsf = translate(next);
            
            //Ales : 
            iReceiver.sendFromPJ(nsf.positions, nsf.lengths, nsf.callsigns);
            nsf.print();
            frames[loops] = nsf;
            
            current_time = next_time;
            prev = next;
            
            loops++;
        }
    }
    
    
    
    
    /*
    private void run_old() {
      // amount of time that passes between two frames.
      final double delta = timescale / fps;
      double next_time = 0;
      FrameFactory.Builder builder = SimMediator.frameFactory.getBuilder(max_craft, 0);
      producer.addInitialAircraft(region, builder);
      SimFrame prev = builder.makeFrame();
      final SerializedFrame sf = translate(prev);

      //TODO:Start.runner.sendFromPJ(sf.positions, sf.lengths, sf.callsigns);

      while (Start.runner.stop == false) {
          next_time = current_time + delta;
          builder = SimMediator.frameFactory.getBuilder(max_craft, next_time);
          addRemainingAircraft(prev, builder);
          final FrameFactory.Mutator mutator = SimMediator.frameFactory.getMutator(builder);
          director.computeNewPositions(region, mutator, delta);
          mutator.commit(); // make sure the changes are finished
          producer.addAircraft(region, prev, builder, delta);
          final SimFrame next = builder.makeFrame();

          final SerializedFrame nsf = translate(next);
          //TODO:Start.runner.sendFromPJ(nsf.positions, nsf.lengths, nsf.callsigns);
          nsf.print();
          
          
          current_time = next_time;
          prev = next;
      }
  }*/

    /**
     * The <code>addRemainingAircraft</code> method is used by the simulator to effectively remove aircraft from a
     * frame. It adds all of the aircraft from the previous frame that will remain and does not add the ones that should
     * be removed. In this way, it effectively removes planes from the next frame of the simulation.
     * @param prev the previous frame of the simulation
     * @param builder the builder to add the new planes to
     */
    private void addRemainingAircraft(final SimFrame prev, final FrameFactory.Builder builder) {
        final SimFrame.PositionIterator iterator = prev.iterator();

        while (iterator.hasNext()) {
            final SimAircraft a = iterator.next(vec0);
            final SimAirport dest = a.getDestination();
            if (isOutsideRegion(vec0)) continue;
            // don't add planes that have reached their destination
            if (dest.contains(vec0)) continue;
            // don't add planes that have crashed into the ground
            if (vec0.z <= 0) continue;
            builder.addPosition(a, vec0);
        }
    }

    private boolean isOutsideRegion(final Vector3d vec) {
        if (vec0.z > SimulatorConstants.MAX_ALTITUDE) return true;
        return !region.getLegalRegion().contains(vec0);
    }

    public static void generate(final String[] args, SerializedFrame[] fr, IReceiver iReceiver) {
        final SimMediator m = new SimMediator();
        final MySimulator s = m.buildSimulator(args[0]);
        
        frames = fr;
        
        s.bindReceiver(iReceiver);
        
        s.run();
    }

		private void bindReceiver(final IReceiver receiver) {
			iReceiver = receiver;
		}
    
    

}
